@@ -1,10 +1,15 @@
 from __future__ import annotations
 
+import asyncio
 import logging
+import random
 import sys
+import warnings
 
 import tornado
 
+__all__ = ["logging_names", "PeriodicCallback", "to_thread", "randbytes"]
+
 logging_names: dict[str | int, int | str] = {}
 logging_names.update(logging._levelToName)  # type: ignore
 logging_names.update(logging._nameToLevel)  # type: ignore
@@ -14,38 +19,22 @@ MACOS = sys.platform == "darwin"
 WINDOWS = sys.platform == "win32"
 
 
-if sys.version_info >= (3, 9):
-    from asyncio import to_thread
-else:
-    import contextvars
-    import functools
-    from asyncio import events
-
-    async def to_thread(func, /, *args, **kwargs):
-        """Asynchronously run function *func* in a separate thread.
-        Any *args and **kwargs supplied for this function are directly passed
-        to *func*. Also, the current :class:`contextvars.Context` is propagated,
-        allowing context variables from the main thread to be accessed in the
-        separate thread.
-
-        Return a coroutine that can be awaited to get the eventual result of *func*.
-
-        backport from
-        https://github.com/python/cpython/blob/3f1ea163ea54513e00e0e9d5442fee1b639825cc/Lib/asyncio/threads.py#L12-L25
-        """
-        loop = events.get_running_loop()
-        ctx = contextvars.copy_context()
-        func_call = functools.partial(ctx.run, func, *args, **kwargs)
-        return await loop.run_in_executor(None, func_call)
+def to_thread(*args, **kwargs):
+    warnings.warn(
+        "to_thread is deprecated and will be removed in a future release; use asyncio.to_thread instead.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return asyncio.to_thread(*args, **kwargs)
 
 
-if sys.version_info >= (3, 9):
-    from random import randbytes
-else:
-    from random import getrandbits
-
-    def randbytes(size):
-        return getrandbits(size * 8).to_bytes(size, "little")
+def randbytes(*args, **kwargs):
+    warnings.warn(
+        "randbytes is deprecated and will be removed in a future release; use random.randbytes instead.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return random.randbytes(*args, **kwargs)
 
 
 if tornado.version_info >= (6, 2, 0, 0):
@@ -59,9 +48,8 @@ else:
     # takes longer than the interval
     import datetime
     import math
-    import random
+    from collections.abc import Awaitable, Callable
     from inspect import isawaitable
-    from typing import Awaitable, Callable
 
     from tornado.ioloop import IOLoop
     from tornado.log import app_log
