@@ -334,7 +334,12 @@ class SyncMethodMixin:
     @property
     def asynchronous(self):
         """Are we running in the event loop?"""
-        return in_async_call(self.loop, default=getattr(self, "_asynchronous", False))
+        try:
+            return in_async_call(
+                self.loop, default=getattr(self, "_asynchronous", False)
+            )
+        except RuntimeError:
+            return False
 
     def sync(self, func, *args, asynchronous=None, callback_timeout=None, **kwargs):
         """Call `func` with `args` synchronously or asynchronously depending on
@@ -420,6 +425,63 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
         return result
 
 
+if sys.version_info >= (3, 10):
+    from asyncio import Event as LateLoopEvent
+else:
+    # In python 3.10 asyncio.Lock and other primitives no longer support
+    # passing a loop kwarg to bind to a loop running in another thread
+    # e.g. calling from Client(asynchronous=False). Instead the loop is bound
+    # as late as possible: when calling any methods that wait on or wake
+    # Future instances. See: https://bugs.python.org/issue42392
+    class LateLoopEvent:
+        _event: asyncio.Event | None
+
+        def __init__(self) -> None:
+            self._event = None
+
+        def set(self) -> None:
+            if self._event is None:
+                self._event = asyncio.Event()
+
+            self._event.set()
+
+        def is_set(self) -> bool:
+            return self._event is not None and self._event.is_set()
+
+        async def wait(self) -> bool:
+            if self._event is None:
+                self._event = asyncio.Event()
+
+            return await self._event.wait()
+
+
+class _CollectErrorThread:
+    def __init__(self, target: Callable[[], None], daemon: bool, name: str):
+        self._exception: BaseException | None = None
+
+        def wrapper() -> None:
+            try:
+                target()
+            except BaseException as e:
+                self._exception = e
+
+        self._thread = thread = threading.Thread(
+            target=wrapper, daemon=daemon, name=name
+        )
+        thread.start()
+
+    def join(self, timeout: float | None = None) -> None:
+        thread = self._thread
+        thread.join(timeout=timeout)
+        if thread.is_alive():
+            raise TimeoutError("join timed out")
+        if self._exception is not None:
+            try:
+                raise self._exception
+            finally:  # remove a reference cycle
+                del self._exception
+
+
 class LoopRunner:
     """
     A helper to start and stop an IO loop in a controlled way.
@@ -442,36 +504,30 @@ class LoopRunner:
         weakref.WeakKeyDictionary[IOLoop, tuple[int, LoopRunner | None]]
     ] = weakref.WeakKeyDictionary()
     _lock = threading.Lock()
+    _loop_thread: _CollectErrorThread | None
 
-    def __init__(self, loop=None, asynchronous=False):
+    def __init__(self, loop: IOLoop | None = None, asynchronous: bool = False):
         if loop is None:
             if asynchronous:
+                # raises RuntimeError if there's no running loop
                 try:
                     asyncio.get_running_loop()
-                except RuntimeError:
-                    warnings.warn(
-                        "Constructing a LoopRunner(asynchronous=True) without a running loop is deprecated",
-                        DeprecationWarning,
-                        stacklevel=2,
-                    )
-                self._loop = IOLoop.current()
-            else:
-                # We're expecting the loop to run in another thread,
-                # avoid re-using this thread's assigned loop
-                self._loop = IOLoop()
-        else:
-            if not loop.asyncio_loop.is_running():
-                warnings.warn(
-                    "Constructing LoopRunner(loop=loop) without a running loop is deprecated",
-                    DeprecationWarning,
-                    stacklevel=2,
-                )
-            self._loop = loop
+                except RuntimeError as e:
+                    raise RuntimeError(
+                        "Constructing LoopRunner(asynchronous=True) without a running loop is not supported"
+                    ) from e
+                loop = IOLoop.current()
+        elif not loop.asyncio_loop.is_running():  # type: ignore[attr-defined]
+            # LoopRunner is not responsible for starting a foreign IOLoop
+            raise RuntimeError(
+                "Constructing LoopRunner(loop=loop) without a running loop is not supported"
+            )
+
+        self._loop = loop
         self._asynchronous = asynchronous
         self._loop_thread = None
         self._started = False
-        with self._lock:
-            self._all_loops.setdefault(self._loop, (0, None))
+        self._stop_event = LateLoopEvent()
 
     def start(self):
         """
@@ -483,62 +539,53 @@ class LoopRunner:
         with self._lock:
             self._start_unlocked()
 
-    def _start_unlocked(self):
+    def _start_unlocked(self) -> None:
         assert not self._started
 
-        count, real_runner = self._all_loops[self._loop]
-        if self._asynchronous or real_runner is not None or count > 0:
+        if self._loop is not None:
+            try:
+                count, real_runner = self._all_loops[self._loop]
+            except KeyError:
+                assert self._loop.asyncio_loop.is_running()  # type: ignore[attr-defined]
+                self._started = True
+                return
+
             self._all_loops[self._loop] = count + 1, real_runner
             self._started = True
             return
 
         assert self._loop_thread is None
-        assert count == 0
 
-        loop_evt = threading.Event()
-        done_evt = threading.Event()
-        in_thread = [None]
-        start_exc = [None]
+        start_evt = threading.Event()
+        start_exc = None
+        loop = None
 
-        def loop_cb():
-            in_thread[0] = threading.current_thread()
-            loop_evt.set()
+        async def amain() -> None:
+            nonlocal loop
+            loop = IOLoop.current()
+            start_evt.set()
+            await self._stop_event.wait()
 
-        def run_loop(loop=self._loop):
-            loop.add_callback(loop_cb)
-            # run loop forever if it's not running already
+        def run_loop() -> None:
+            nonlocal start_exc
             try:
-                if not loop.asyncio_loop.is_running():
-                    loop.start()
-            except Exception as e:
-                start_exc[0] = e
-            finally:
-                done_evt.set()
+                asyncio.run(amain())
+            except BaseException as e:
+                if start_evt.is_set():
+                    raise
+                start_exc = e
+                start_evt.set()
 
-        thread = threading.Thread(target=run_loop, name="IO loop")
-        thread.daemon = True
-        thread.start()
-
-        loop_evt.wait(timeout=10)
+        self._loop_thread = _CollectErrorThread(
+            target=run_loop, daemon=True, name="IO loop"
+        )
+        start_evt.wait(timeout=10)
+        if start_exc is not None:
+            raise start_exc
+        assert loop is not None
+        self._loop = loop
         self._started = True
-
-        actual_thread = in_thread[0]
-        if actual_thread is not thread:
-            # Loop already running in other thread (user-launched)
-            done_evt.wait(5)
-            if start_exc[0] is not None and not isinstance(start_exc[0], RuntimeError):
-                if not isinstance(
-                    start_exc[0], Exception
-                ):  # track down infrequent error
-                    raise TypeError(
-                        f"not an exception: {start_exc[0]!r}",
-                    )
-                raise start_exc[0]
-            self._all_loops[self._loop] = count + 1, None
-        else:
-            assert start_exc[0] is None, start_exc
-            self._loop_thread = thread
-            self._all_loops[self._loop] = count + 1, self
+        self._all_loops[loop] = (1, self)
 
     def stop(self, timeout=10):
         """
@@ -554,25 +601,26 @@ class LoopRunner:
 
         self._started = False
 
-        count, real_runner = self._all_loops[self._loop]
+        try:
+            count, real_runner = self._all_loops[self._loop]
+        except KeyError:
+            return
+
         if count > 1:
             self._all_loops[self._loop] = count - 1, real_runner
-        else:
-            assert count == 1
-            del self._all_loops[self._loop]
-            if real_runner is not None:
-                real_runner._real_stop(timeout)
+            return
+
+        assert count == 1
+        del self._all_loops[self._loop]
+        real_runner._real_stop(timeout)
 
     def _real_stop(self, timeout):
         assert self._loop_thread is not None
-        if self._loop_thread is not None:
-            try:
-                self._loop.add_callback(self._loop.stop)
-                self._loop_thread.join(timeout=timeout)
-                with contextlib.suppress(KeyError):  # IOLoop can be missing
-                    self._loop.close()
-            finally:
-                self._loop_thread = None
+        try:
+            self._loop.add_callback(self._stop_event.set)
+            self._loop_thread.join(timeout=timeout)
+        finally:
+            self._loop_thread = None
 
     def is_started(self):
         """
@@ -597,11 +645,9 @@ class LoopRunner:
     @property
     def loop(self):
         loop = self._loop
-        if not loop.asyncio_loop.is_running():
-            warnings.warn(
-                "Accessing the loop property while the loop is not running is deprecated",
-                DeprecationWarning,
-                stacklevel=2,
+        if loop is None or not loop.asyncio_loop.is_running():
+            raise RuntimeError(
+                "Accessing the loop property while the loop is not running is not supported"
             )
         return self._loop
 
